//------------------------------------------------------------------------------
//
//  File:  gpio.h
//
#ifndef __GPIO_H
#define __GPIO_H
extern "C"
{
	BOOL KernelIoControl(DWORD dwIoControlCode, LPVOID lpInBuf, DWORD nInBufSize, LPVOID lpOutBuf, DWORD nOutBufSize, LPDWORD lpBytesReturned);
}

#define GPIO_IOCTL_INDEX                            0x0810


#define IOCTL_GPIO_SET_CURRENT							CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 0,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_SET_DIR								CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 1,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_SET_INTERRUPT_FALLING_EDGE		    CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 2,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)


#define IOCTL_GPIO_SET_INTERRUPT_RISING_EDGE            CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 3,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_REQUEST_INTERRUPT_EVENT		        CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 4,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_RELEASE_INTERRUPT_EVENT		        CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 5,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)


#define IOCTL_GPIO_SET_INTERRUPT						CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 6,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_RELEASE_INTERRUPT					CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 7,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_SET_VALUE							CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 8,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_GET_VALUE							CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 9,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_INTERRUPT_DONE						CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 10,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_ACQUIRE								CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 11,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_RELEASE								CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 12,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_GET_FREE_STATUS						CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 13,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_GET_DRIVER_IFC						CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 14,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)

#define IOCTL_GPIO_SET_ALTERNATE_FUNC					CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        GPIO_IOCTL_INDEX + 15,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)


#define GPIO_NOT_AVAILABLE		0xa

typedef enum
{
	GPIO_REACT_ON_FALLING,
	GPIO_REACT_ON_RISING,
	GPIO_REACT_ON_BOTH,
	GPIO_DONT_REACT
} GPIO_EDGE_SETTINGS;

typedef enum
{
	GPIO_INPUT,
	GPIO_OUTPUT
}GPIO_DIRECTION;

typedef enum
{
	GPIO_LOW,
	GPIO_HIGH
}GPIO_VALUE;

typedef struct gpio_intr_t{
	DWORD				dwGpioNum;
	GPIO_EDGE_SETTINGS	Edge;
	HANDLE				Event;
} GPIO_INTERRUPT;

typedef enum
{
	AF_GPIO,	//Regular GPIO
	AF_1,		//Alterante function 1
	AF_2,		//Alterante function 2
	AF_3		//Alterante function 3
}GPIO_AF;

typedef struct gpio_af_t{
	DWORD			dwGpioNum;
	GPIO_AF			af;	
} GPIO_AF_DESCRIPTOR;

typedef struct {
    BOOL (*pfnSetAsOutput)(DWORD num, GPIO_VALUE val);
    BOOL (*pfnSetAsInput)(DWORD num);
	BOOL (*pfnRegisterInterrupt)(GPIO_INTERRUPT interrupt);
	BOOL (*pfnUnRegisterInterrupt)(DWORD num);
	BOOL (*pfnGetValue)(DWORD num, GPIO_VALUE *gpioVal);
	BOOL (*pfnSetValue)(DWORD num, GPIO_VALUE gpioVal);
	BOOL (*pfnInterruptDone)(DWORD num);
	BOOL (*pfnIsAvailable)(DWORD num);
	BOOL (*pfnAcquire)(DWORD num);
	BOOL (*pfnRelease)(DWORD num);
	BOOL (*pfnSetAF)(DWORD dwGpioNum, GPIO_AF af);
} DEVICE_IFC_GPIO;

typedef struct gpio_state_t{
	DWORD			dwGpioNum;
	GPIO_DIRECTION  Dir;	//gpio direction
	GPIO_VALUE		Val;	//ignored if gpio's direction isn't output
} GPIO_STATE;


__inline HANDLE GPIOOpen()
{
    HANDLE hDevice;
	hDevice = CreateFile (L"GIO1:", 0, 0, NULL, OPEN_ALWAYS, 0, NULL);
    return hDevice;
}

__inline VOID GPIOClose(HANDLE hContext)
{
    if(hContext == NULL)
		return;
	CloseHandle(hContext);
}

__inline BOOL GPIOSetAsOutput(HANDLE hDevice, DWORD num, GPIO_VALUE val)
{
	BOOL iRet;
	GPIO_STATE state;
	if(hDevice == NULL)
		return FALSE;

	state.Dir = GPIO_OUTPUT;
	state.dwGpioNum = num;
	state.Val = val;
	iRet = DeviceIoControl (hDevice, IOCTL_GPIO_SET_VALUE, &state, sizeof(GPIO_STATE), NULL, 0, NULL, NULL);
	return iRet;
}

__inline BOOL GPIOSetAsInput(HANDLE hDevice, DWORD num)
{
	BOOL iRet;
	GPIO_STATE state;
	if(hDevice == NULL)
		return FALSE;

	state.Dir = GPIO_INPUT;
	state.dwGpioNum = num;
	iRet = DeviceIoControl (hDevice, IOCTL_GPIO_SET_VALUE, &state, sizeof(GPIO_STATE), NULL, 0, NULL, NULL);
	return iRet;
}

__inline BOOL GPIORegisterInterrupt(HANDLE hDevice, GPIO_INTERRUPT interrupt)
{
	BOOL iRet;
	if(hDevice == NULL)
		return FALSE;
	iRet = DeviceIoControl(hDevice, IOCTL_GPIO_SET_INTERRUPT, &interrupt, sizeof(GPIO_INTERRUPT), NULL, 0, NULL, NULL);
	return iRet;
}

__inline BOOL GPIOUnRegisterInterrupt(HANDLE hDevice, DWORD num)
{
	BOOL iRet;
	if(hDevice == NULL)
		return FALSE;
	iRet = DeviceIoControl(hDevice, IOCTL_GPIO_RELEASE_INTERRUPT, &num, sizeof(DWORD), NULL, 0, NULL, NULL);
	return iRet;
}

__inline BOOL GPIOGetValue(HANDLE hDevice, DWORD num, GPIO_VALUE *gpioVal)
{
	BYTE val;
	BOOL iRet;
	if(hDevice == NULL)
		return FALSE;
	iRet = DeviceIoControl(hDevice, IOCTL_GPIO_GET_VALUE, &num, sizeof(DWORD), &val, sizeof(BYTE), NULL, NULL);
	*gpioVal = (val) ? GPIO_HIGH : GPIO_LOW;
	return iRet;
}

__inline BOOL GPIOSetValue(HANDLE hDevice, DWORD num, GPIO_VALUE val)
{
	//sets a specific value to the given GPIO
	if(hDevice == NULL)
		return FALSE;
	return GPIOSetAsOutput(hDevice, num, val);
}

__inline void GPIOInterruptDone(HANDLE hDevice, DWORD num)
{
	if(hDevice == NULL)
		return;
	DeviceIoControl(hDevice, IOCTL_GPIO_INTERRUPT_DONE, &num, sizeof(DWORD), NULL, 0, NULL, NULL);
}


__inline BOOL GPIOIsAvailable(HANDLE hDevice, DWORD num)
{
	BOOL bIsAvailable, iRet;
	if(hDevice == NULL)
		return FALSE;
	iRet = DeviceIoControl(hDevice, IOCTL_GPIO_GET_FREE_STATUS, &num, sizeof(DWORD), &bIsAvailable, sizeof(BOOL), NULL, NULL);
	return ((iRet) ? bIsAvailable : FALSE);
}

__inline BOOL GPIOAquireGpio(HANDLE hDevice, DWORD num)
{
	BOOL iRet;
	if(hDevice == NULL)
		return FALSE;
	iRet = DeviceIoControl(hDevice, IOCTL_GPIO_ACQUIRE, &num, sizeof(DWORD), NULL, 0 , NULL, NULL);
	return iRet;
}

__inline BOOL GPIOReleaseGpio(HANDLE hDevice, DWORD num)
{
	BOOL iRet;
	if(hDevice == NULL)
		return FALSE;
	iRet = DeviceIoControl(hDevice, IOCTL_GPIO_RELEASE, &num, sizeof(DWORD), NULL, 0 , NULL, NULL);
	return iRet;
}

__inline BOOL GPIOSetAlternateFunc(HANDLE hDevice, DWORD num, GPIO_AF af)
{
	GPIO_AF_DESCRIPTOR afd;
	if(hDevice == NULL)
		return FALSE;
	afd.af = af;
	afd.dwGpioNum = num;
	return DeviceIoControl(hDevice, IOCTL_GPIO_SET_ALTERNATE_FUNC, &afd, sizeof(GPIO_AF_DESCRIPTOR), NULL, 0, NULL, NULL);

}

#endif
